@@ -58,6 +58,65 @@ def median_absolute_error(A, F):
     return mae_result
 
 
+def mean_absolute_differential_error(A, F, order: int = 1, df_train=None, scaler=None):
+    """Expects two, 2-D numpy arrays of forecast_length * n series.
+
+    Returns a 1-D array of results in len n series
+
+    Args:
+        A (numpy.array): known true values
+        F (numpy.array): predicted values
+        order (int): order of differential
+        df_train (np.array): if provided, uses this as starting point for first diff step.
+            Tail(1) must be most recent historical point before forecast.
+            Must be numpy Array not DataFrame.
+            Highly recommended if using this as the sole optimization metric.
+            Without, it is an "unanchored" shape fitting metric.
+            This will also allow this to work on forecast_length = 1 forecasts
+        scaler (np.array): if provided, metrics are scaled by this. 1d array of shape (num_series,)
+    """
+    # scaler = np.mean(A, axis=0)  # debate over whether to make this scaled
+    if df_train is not None:
+        last_of_array = np.nan_to_num(
+            df_train[
+                df_train.shape[0] - 1 : df_train.shape[0],
+            ]
+        )
+        # last_of_array = df_train.tail(1).fillna(0).to_numpy()
+        # assigning to new because I'm paranoid about overwrite existing objects
+        lA = np.concatenate([last_of_array, A])
+        lF = np.concatenate([last_of_array, F])
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", category=RuntimeWarning)
+            if scaler is None:
+                return np.nanmean(
+                    abs(np.diff(lA, order, axis=0) - np.diff(lF, order, axis=0)), axis=0
+                )
+            else:
+                return (
+                    np.nanmean(
+                        abs(np.diff(lA, order, axis=0) - np.diff(lF, order, axis=0)),
+                        axis=0,
+                    )
+                    / scaler
+                )
+    else:
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", category=RuntimeWarning)
+            if scaler is None:
+                return np.nanmean(
+                    abs(np.diff(A, order, axis=0) - np.diff(F, order, axis=0)), axis=0
+                )
+            else:
+                return (
+                    np.nanmean(
+                        abs(np.diff(A, order, axis=0) - np.diff(F, order, axis=0)),
+                        axis=0,
+                    )
+                    / scaler
+                )
+
+
 def pinball_loss(A, F, quantile):
     """Bigger is bad-er."""
     with warnings.catch_warnings():
@@ -132,29 +191,47 @@ def contour(A, F):
     Expects two, 2-D numpy arrays of forecast_length * n series
     Returns a 1-D array of results in len n series
 
+    Concat the last row of history to head of both A and F (req for 1 step)
+
     Args:
         A (numpy.array): known true values
         F (numpy.array): predicted values
     """
 
-    with warnings.catch_warnings():
-        warnings.simplefilter("ignore", category=RuntimeWarning)
-        try:
-            X = np.nan_to_num(np.diff(A, axis=0))
-            Y = np.nan_to_num(np.diff(F, axis=0))
-            # On the assumption flat lines common in forecasts,
-            # but exceedingly rare in real world
-            X = X >= 0
-            Y = Y > 0
-            contour_result = np.sum(X == Y, axis=0) / X.shape[0]
-        except Exception:
-            contour_result = np.nan
+    try:
+        X = np.nan_to_num(np.diff(A, axis=0))
+        Y = np.nan_to_num(np.diff(F, axis=0))
+        # On the assumption flat lines common in forecasts,
+        # but exceedingly rare in real world
+        X = X >= 0
+        Y = Y > 0
+        contour_result = np.sum(X == Y, axis=0) / X.shape[0]
+    except Exception:
+        contour_result = np.nan
     return contour_result
 
 
-def rmse(ae):
-    """Accepting abs error already calculated"""
-    return np.sqrt(np.nanmean((ae ** 2), axis=0))
+def rps(predictions, observed):
+    """Vectorized version of Ranked Probability Score.
+    A lower value is a better score.
+    From: Colin Catlin, https://syllepsis.live/2022/01/22/ranked-probability-score-in-python/
+
+    Args:
+        predictions (pd.DataFrame): each column is an outcome category, with values as the 0 to 1 probability of that category
+        observed (pd.DataFrame): each column is an outcome category, with values of 0 OR 1 with 1 being that category occurred
+    """
+    ncat = predictions.shape[1] - 1
+    return (
+        np.sum(
+            (np.cumsum(predictions, axis=1) - np.cumsum(observed, axis=1)) ** 2, axis=1
+        )
+        / ncat
+    )
+
+
+def rmse(sqe):
+    """Accepting squared error already calculated"""
+    return np.sqrt(np.nanmean(sqe, axis=0))
 
 
 def mae(ae):
@@ -174,7 +251,7 @@ def smape(actual, forecast, ae):
     ) / np.count_nonzero(~np.isnan(actual), axis=0)
 
 
-def spl(A, F, quantile, scaler):
+def _spl(A, F, quantile, scaler):
     """Accepting scaler already calculated"""
     return (
         np.nanmean(
@@ -182,3 +259,16 @@ def spl(A, F, quantile, scaler):
         )
         / scaler
     )
+
+
+def spl(precomputed_spl, scaler):
+    """Accepting most of it already calculated"""
+    return np.nanmean(precomputed_spl, axis=0) / scaler
+
+
+def msle(full_errors, ae, le):
+    """input is array of y_pred - y_true to over-penalize underestimate.
+    Use instead y_true - y_pred to over-penalize overestimate.
+    AE used here for the log just to avoid divide by zero warnings (values aren't used either way)
+    """
+    return np.nanmean(np.where(full_errors > 0, le, ae), axis=0)
