@@ -30,6 +30,8 @@ all_models = [
     'SectionalMotif',
     'Theta',
     'ARDL',
+    'NeuralProphet',
+    'DynamicFactorMQ',
 ]
 default = [
     'ZeroesNaive',
@@ -47,14 +49,42 @@ default = [
     'VECM',
     'WindowRegression',
     'DatepartRegression',
-    'UnivariateRegression',
-    # 'MultivariateRegression',  # downweight if that becomes an option
+    # 'UnivariateRegression',  # this has been crashing on 1135
+    'MultivariateRegression',  # downweight if that becomes an option
     'UnivariateMotif',
     'MultivariateMotif',
     'SectionalMotif',
     'NVAR',
     'Theta',
     'ARDL',
+    # 'DynamicFactorMQ',
+]
+colin = [
+    'LastValueNaive',
+    'AverageValueNaive',
+    'GLS',
+    'GLM',
+    'ETS',
+    # 'ARIMA',
+    'FBProphet',
+    # 'RollingRegression',
+    'GluonTS',
+    'SeasonalNaive',
+    'UnobservedComponents',
+    # 'VARMAX',
+    'VECM',
+    # 'MotifSimulation',
+    # 'UnivariateRegression',
+    'MultivariateRegression',
+    'WindowRegression',
+    'VAR',
+    'DatepartRegression',
+    'UnivariateMotif',
+    'MultivariateMotif',
+    'NVAR',
+    'SectionalMotif',
+    'Theta',
+    'ARDL',
 ]
 # fastest models at any scale
 superfast = [
@@ -125,6 +155,7 @@ probabilistic = [
     'Theta',
     'ARDL',
     'UnobservedComponents',
+    'DynamicFactorMQ',
     # 'MultivariateRegression',
 ]
 # models that use the shared information of multiple series to improve accuracy
@@ -140,6 +171,7 @@ multivariate = [
     'NVAR',
     'MultivariateRegression',
     'SectionalMotif',
+    'DynamicFactorMQ',
 ]
 univariate = list((set(all_models) - set(multivariate)) - set(experimental))
 # USED IN AUTO_MODEL, models with no parameters
@@ -170,6 +202,8 @@ recombination_approved = [
     'SectionalMotif',
     'Theta',
     'ARDL',
+    'NeuralProphet',
+    'DynamicFactorMQ',
 ]
 # USED IN AUTO_MODEL for models that don't share information among series
 no_shared = [
@@ -189,6 +223,7 @@ no_shared = [
     'UnivariateMotif',
     'Theta',
     'ARDL',
+    'NeuralProphet',
 ]
 # allow the use of a regressor, need to accept "User" (fail if not given), have 'regressor' param method
 regressor = [
@@ -207,6 +242,7 @@ regressor = [
     'MultivariateRegression',
     'SectionalMotif',  # kinda
     'ARDL',
+    'NeuralProphet',
 ]
 no_shared_fast = list(set(no_shared).intersection(set(fast_parallel)))
 model_lists = {
@@ -227,6 +263,7 @@ model_lists = {
     "slow": slow,
     "gpu": gpu,
     "regressor": regressor,
+    "colin": colin,
 }
 
 
