@@ -470,7 +470,8 @@ class ETS(ModelObject):
             """Run one series of ETS and return prediction."""
             series_name = current_series.name
             with warnings.catch_warnings():
-                warnings.simplefilter("ignore", category='ConvergenceWarning')
+                if args['verbose'] < 2:
+                    warnings.simplefilter("ignore")
                 try:
                     # handle statsmodels 0.13 method changes
                     try:
@@ -480,10 +481,12 @@ class ETS(ModelObject):
                             trend=args['trend'],
                             seasonal=args['seasonal'],
                             seasonal_periods=args['seasonal_periods'],
-                            initialization_method=None,
-                            freq=args['freq'],
+                            # initialization_method=None,
+                            # freq=args['freq'],
                         )
-                    except Exception:
+                    except Exception as e:
+                        if args['verbose'] > 0:
+                            print(f"ETS error {repr(e)}")
                         esModel = ExponentialSmoothing(
                             current_series,
                             damped=args['damped_trend'],
@@ -500,7 +503,8 @@ class ETS(ModelObject):
                     )
                     esPred = pd.Series(esPred)
                 except Exception as e:
-                    if args['verbose'] > 1:
+                    # this error handling is meant for horizontal ensembles where it will only then be needed for select series
+                    if args['verbose'] > 0:
                         print(f"ETS failed on {series_name} with {repr(e)}")
                     esPred = pd.Series((np.zeros((forecast_length,))), index=test_index)
             esPred.name = series_name
@@ -588,7 +592,8 @@ class ETS(ModelObject):
 def arima_seek_the_oracle(current_series, args, series):
     try:
         with warnings.catch_warnings():
-            warnings.simplefilter("ignore", category='ConvergenceWarning')
+            # warnings.simplefilter("ignore", category='ConvergenceWarning')
+            warnings.simplefilter("ignore")
             if args['regression_type'] in ["User", "Holiday"]:
                 maModel = SARIMAX(
                     current_series,
@@ -854,7 +859,7 @@ class UnobservedComponents(ModelObject):
         random_seed: int = 2020,
         verbose: int = 0,
         n_jobs: int = 1,
-        level: bool = False,
+        level: str = "smooth trend",  # bool with other method
         trend: bool = False,
         cycle: bool = False,
         damped_cycle: bool = False,
@@ -974,13 +979,12 @@ class UnobservedComponents(ModelObject):
             'model_kwargs': self.model_kwargs,
         }
 
-        def forecast_by_column(current_series, args):
+        def uc_forecast_by_column(current_series, args):
             """Run one series of Unobserved Components and return prediction."""
             series_name = current_series.name
             with warnings.catch_warnings():
-                warnings.simplefilter("ignore", category='ConvergenceWarning')
-                warnings.simplefilter("ignore", category='ValueWarning')
-                warnings.simplefilter("ignore", category='UserWarning')
+                if not args['verbose_bool']:
+                    warnings.simplefilter("ignore")
                 if args['regression_type'] in ["User", "Holiday"]:
                     maModel = UnobservedComponents(
                         current_series,
@@ -1043,7 +1047,7 @@ class UnobservedComponents(ModelObject):
         if parallel:
             verbs = 0 if self.verbose < 1 else self.verbose - 1
             df_list = Parallel(n_jobs=self.n_jobs, verbose=(verbs))(
-                delayed(forecast_by_column)(
+                delayed(uc_forecast_by_column)(
                     current_series=self.df_train[col],
                     args=args,
                 )
@@ -1053,7 +1057,7 @@ class UnobservedComponents(ModelObject):
         else:
             df_list = []
             for col in cols:
-                df_list.append(forecast_by_column(self.df_train[col], args))
+                df_list.append(uc_forecast_by_column(self.df_train[col], args))
             complete = list(map(list, zip(*df_list)))
         forecast = pd.concat(complete[0], axis=1)
         lower_forecast = pd.concat(complete[1], axis=1)
@@ -1315,21 +1319,19 @@ class DynamicFactor(ModelObject):
 
     def get_new_params(self, method: str = 'random'):
         """Return dict of new parameters for parameter tuning."""
-        k_factors_choice = np.random.choice(
-            a=[0, 1, 2, 3, 10], size=1, p=[0.1, 0.4, 0.2, 0.2, 0.1]
-        ).item()
-        factor_order_choice = np.random.choice(
-            a=[0, 1, 2, 3], size=1, p=[0.4, 0.3, 0.2, 0.1]
-        ).item()
+        k_factors_choice = random.choices([0, 1, 2, 3, 10], [0.1, 0.4, 0.2, 0.2, 0.1])[
+            0
+        ]
+        factor_order_choice = random.choices([0, 1, 2, 3], [0.4, 0.3, 0.2, 0.1])[0]
 
         if "regressor" in method:
             regression_choice = "User"
         else:
             regression_list = [None, 'User', 'Holiday']
             regression_probability = [0.6, 0.2, 0.2]
-            regression_choice = np.random.choice(
-                a=regression_list, size=1, p=regression_probability
-            ).item()
+            regression_choice = random.choices(regression_list, regression_probability)[
+                0
+            ]
 
         parameter_dict = {
             'k_factors': k_factors_choice,
@@ -1369,7 +1371,7 @@ class VECM(ModelObject):
         holiday_country: str = 'US',
         random_seed: int = 2020,
         verbose: int = 0,
-        deterministic: str = 'nc',
+        deterministic: str = 'n',
         k_ar_diff: int = 1,
         **kwargs,
     ):
@@ -1491,7 +1493,7 @@ class VECM(ModelObject):
     def get_new_params(self, method: str = 'random'):
         """Return dict of new parameters for parameter tuning."""
         deterministic_choice = np.random.choice(
-            a=["nc", "co", "ci", "lo", "li", "cili", "colo"],
+            a=["n", "co", "ci", "lo", "li", "cili", "colo"],
             size=1,
             p=[0.4, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1],
         ).item()
@@ -1762,7 +1764,7 @@ class VAR(ModelObject):
         if self.regression_type in ["User", "Holiday"]:
             maModel = VAR(
                 self.df_train, freq=self.frequency, exog=self.regressor_train
-            ).fit(maxlags=15, ic='fpe', trend='nc')
+            ).fit(maxlags=self.maxlags, ic=self.ic, trend='n')
             forecast, lower_forecast, upper_forecast = maModel.forecast_interval(
                 steps=len(test_index),
                 exog_future=future_regressor,
@@ -1927,6 +1929,7 @@ class Theta(ModelObject):
             'prediction_interval': self.prediction_interval,
             'theta': self.theta,
             'use_mle': self.use_mle,
+            'verbose': self.verbose,
         }
 
         def theta_forecast_by_column(current_series, args):
@@ -1941,8 +1944,9 @@ class Theta(ModelObject):
                 period=args['period'],
             )
             with warnings.catch_warnings():
-                warnings.simplefilter("ignore", category='ConvergenceWarning')
-                warnings.simplefilter("ignore", category=UserWarning)
+                # warnings.simplefilter("ignore", category='ConvergenceWarning')
+                if args['verbose'] < 2:
+                    warnings.simplefilter("ignore")
                 # fit model
                 modelResult = esModel.fit(use_mle=args['use_mle'])
                 # generate forecasts
@@ -2127,9 +2131,10 @@ class ARDL(ModelObject):
 
         def ardl_per_column(current_series, args):
             with warnings.catch_warnings():
-                warnings.simplefilter("ignore", category='ConvergenceWarning')
-                warnings.simplefilter("ignore", category='ValueWarning')
-                warnings.simplefilter("ignore", category='UserWarning')
+                # warnings.simplefilter("ignore", category='ConvergenceWarning')
+                # warnings.simplefilter("ignore", category='ValueWarning')
+                if args['verbose'] < 2:
+                    warnings.simplefilter("ignore", category=UserWarning)
                 if args['regression_type'] in ["User", "user", "holiday"]:
                     maModel = ARDL(
                         current_series,
@@ -2187,6 +2192,7 @@ class ARDL(ModelObject):
             'trend': self.trend,
             'alpha': alpha,
             'forecast_length': forecast_length,
+            'verbose': self.verbose,
         }
         parallel = True
         cols = self.df_train.columns.tolist()
@@ -2237,9 +2243,9 @@ class ARDL(ModelObject):
     def get_new_params(self, method: str = 'random'):
         """Return dict of new parameters for parameter tuning."""
         if "regressor" in method:
-            regression_choice = "user"
+            regression_choice = "User"
         else:
-            regression_list = [None, 'user', 'holiday']
+            regression_list = [None, 'User', 'holiday']
             regression_probability = [0.3, 0.5, 0.5]
             regression_choice = random.choices(regression_list, regression_probability)[
                 0
@@ -2264,3 +2270,145 @@ class ARDL(ModelObject):
             'order': self.order,
             'regression_type': self.regression_type,
         }
+
+
+class DynamicFactorMQ(ModelObject):
+    """DynamicFactorMQ from Statsmodels
+
+    Args:
+        name (str): String to identify class
+        frequency (str): String alias of datetime index frequency or else 'infer'
+        prediction_interval (float): Confidence interval for probabilistic forecast
+
+    """
+
+    def __init__(
+        self,
+        name: str = "DynamicFactorMQ",
+        frequency: str = 'infer',
+        prediction_interval: float = 0.9,
+        regression_type: str = None,
+        holiday_country: str = 'US',
+        random_seed: int = 2020,
+        verbose: int = 0,
+        factors: int = 1,
+        factor_orders: int = 2,
+        factor_multiplicities: int = None,
+        idiosyncratic_ar1: bool = False,
+        maxiter: int = 1000,
+        **kwargs,
+    ):
+        ModelObject.__init__(
+            self,
+            name,
+            frequency,
+            prediction_interval,
+            regression_type=regression_type,
+            holiday_country=holiday_country,
+            random_seed=random_seed,
+            verbose=verbose,
+        )
+        self.factors = factors
+        self.factor_orders = factor_orders
+        self.factor_multiplicities = factor_multiplicities
+        self.idiosyncratic_ar1 = idiosyncratic_ar1
+        self.maxiter = maxiter
+
+    def fit(self, df, future_regressor=None):
+        """Train algorithm given data supplied.
+
+        Args:
+            df (pandas.DataFrame): Datetime Indexed
+        """
+        df = self.basic_profile(df)
+        if self.verbose > 2:
+            self.verbose = True
+        else:
+            self.verbose = False
+        self.df_train = df
+
+        self.fit_runtime = datetime.datetime.now() - self.startTime
+        return self
+
+    def predict(
+        self, forecast_length: int, future_regressor=None, just_point_forecast=False
+    ):
+        """Generates forecast data immediately following dates of index supplied to .fit()
+
+        Args:
+            forecast_length (int): Number of periods of data to forecast ahead
+            regressor (numpy.Array): additional regressor
+            just_point_forecast (bool): If True, return a pandas.DataFrame of just point forecasts
+
+        Returns:
+            Either a PredictionObject of forecasts and metadata, or
+            if just_point_forecast == True, a dataframe of point forecasts
+        """
+        predictStartTime = datetime.datetime.now()
+        from statsmodels.tsa.statespace.dynamic_factor_mq import DynamicFactorMQ
+
+        test_index = self.create_forecast_index(forecast_length=forecast_length)
+
+        maModel = DynamicFactorMQ(
+            self.df_train,
+            freq=self.frequency,
+            factors=self.factors,
+            factor_orders=self.factor_orders,
+            factor_multiplicities=self.factor_multiplicities,
+            idiosyncratic_ar1=self.idiosyncratic_ar1,
+        ).fit(disp=self.verbose, maxiter=self.maxiter)
+        forecast = maModel.predict(start=test_index[0], end=test_index[-1])
+
+        if just_point_forecast:
+            return forecast
+        else:
+            # outer forecasts
+            alpha = 1 - self.prediction_interval
+            # predict_results = maModel.get_prediction(start='2020',end='2021')
+            outer_forecasts = maModel.get_forecast(steps=forecast_length)
+            outer_forecasts_df = outer_forecasts.conf_int(alpha=alpha)
+            df_size = int(outer_forecasts_df.shape[1] / 2)
+            lower_df = outer_forecasts_df.iloc[:, 0:df_size]
+            lower_df = lower_df.rename(columns=lambda x: x[6:])
+            upper_df = outer_forecasts_df.iloc[:, df_size:]
+            upper_df = upper_df.rename(columns=lambda x: x[6:])
+
+            predict_runtime = datetime.datetime.now() - predictStartTime
+            prediction = PredictionObject(
+                model_name=self.name,
+                forecast_length=forecast_length,
+                forecast_index=test_index,
+                forecast_columns=forecast.columns,
+                lower_forecast=lower_df,
+                forecast=forecast,
+                upper_forecast=upper_df,
+                prediction_interval=self.prediction_interval,
+                predict_runtime=predict_runtime,
+                fit_runtime=self.fit_runtime,
+                model_parameters=self.get_params(),
+            )
+
+            return prediction
+
+    def get_new_params(self, method: str = 'random'):
+        """Return dict of new parameters for parameter tuning."""
+        k_factors_choice = random.choices([1, 2, 3, 10], [0.4, 0.2, 0.2, 0.1])[0]
+        factor_order_choice = random.choices([1, 2, 3, 4], [0.3, 0.2, 0.1, 0.02])[0]
+
+        parameter_dict = {
+            'factors': k_factors_choice,
+            'factor_orders': factor_order_choice,
+            "factor_multiplicities": random.choice([None, 2]),
+            'idiosyncratic_ar1': random.choice([True, False]),
+        }
+        return parameter_dict
+
+    def get_params(self):
+        """Return dict of current parameters."""
+        parameter_dict = {
+            'factors': self.factors,
+            'factor_orders': self.factor_orders,
+            'factor_multiplicities': self.factor_multiplicities,
+            'idiosyncratic_ar1': self.idiosyncratic_ar1,
+        }
+        return parameter_dict
