@@ -82,12 +82,14 @@ def create_regressor(
         fill_na=fill_na,
     )
     # datepart
-    if datepart_method in ['simple', 'expanded', 'recurring']:
+    if datepart_method in ['simple', 'expanded', 'recurring', "simple_2"]:
         regr_train = pd.concat(
-            [regr_train, date_part(regr_train.index, method=datepart_method)], axis=1,
+            [regr_train, date_part(regr_train.index, method=datepart_method)],
+            axis=1,
         )
         regr_fcst = pd.concat(
-            [regr_fcst, date_part(regr_fcst.index, method=datepart_method)], axis=1,
+            [regr_fcst, date_part(regr_fcst.index, method=datepart_method)],
+            axis=1,
         )
     # holiday (list)
     if holiday_countries is not None:
